#!/usr/bin/perl -w

# Linux Application Checker
# Stop Process Tree Module (stop_tests.pl)
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

my $kill_pid = shift;
my $server_log = shift;
if (!$kill_pid) {
	die "Usage: $0 <PID> [<SERVER_LOG>]\n<SERVER_LOG> should be specified only when starting from Linux AppChecker!\n";
}

sub is_process_running($) {
	my ($PID) = @_;
	my $test_line = `ps -p $PID -o s --no-heading`;
	$test_line =~ s/[\r\n]//g if ($test_line ne "");
	return (($test_line ne "") and ($test_line !~ m/^Z/i));
}

# Build a tree of processes that will be used for killing the sub-tree.
# The top of the tree is atk-server.pl.
# Sub-tree of this particular process is excluded so that it could finish its work.
my %CHLD = ();
sub build_tree() {
	my %PARENT = ();
	foreach (split(/[\r\n]+/, `ps -A -o pid= -o ppid=`)) {
		s/^\s+//g;
		my ($ps_pid, $ps_ppid) = split(/\s+/, $_, 2);
		if (!defined($CHLD{$ps_ppid})) {
			$CHLD{$ps_ppid} = [];
		}
		push @{$CHLD{$ps_ppid}}, $ps_pid;
		$PARENT{$ps_pid} = $ps_ppid;
	}
	# Removing our subtree from the tree (if present), so that we could live long enough
	# to send the responce.
	# If the script was run from ATK, remove the whole branch; else only this process.
	my $root_pid;
	if (defined($server_log)) {
		`head -n 1 $server_log 2>/dev/null` =~ m/PID: (\d+)\.$/i;
		$root_pid = $1;
	}
	else {
		$root_pid = $PARENT{$$};
	}
	my $pid = $$;
	my $ppid;
	while ($pid) {
		$ppid = $PARENT{$pid};
		if ($ppid ne $root_pid) {
			# Removing the whole sub-tree
			$CHLD{$ppid} = [];
		}
		else {
			#Were on the top; remove only this particular child...
			my $temp = $CHLD{$ppid};
			for (my $i=0; $i<scalar(@$temp); $i++) {
				if ($temp->[$i] eq $pid) {
					splice @$temp, $i, 1;
					last;
				}
			}
			# ... and exit.
			last;
		}
		$pid = $ppid;
	}
}

my %to_check_kill = ();
sub kill_tree($) {
	my $pid = $_[0];
	return if (!$pid or ($pid == getppid()));
	kill("SIGTERM", $pid);
	$to_check_kill{$pid} = 1;
	my $temp = $CHLD{$pid};
	return if (!$temp);
	foreach my $i (@$temp) {
		&kill_tree($i);
	}
}

sub is_tree_killed() {
	# Try 10 times, waiting 0.3 seconds each time, for all the process to be really killed
	for (my $i=0; $i<10; $i++) {
		foreach (keys %to_check_kill) {
			if (!is_process_running($_)) {
				delete $to_check_kill{$_};
			}
		}
		if (scalar(keys %to_check_kill) == 0) {
			return 1;
		}
		else {
			select(undef, undef, undef, 0.3);
		}
	}
	return 0;
}

build_tree();
kill_tree($kill_pid);
if (is_tree_killed()) {
	exit 0;
}
else {
	print STDERR "Could not terminate processes: ".join(", ", sort keys %to_check_kill)."\n";
	exit 1;
}
