#!/usr/bin/perl -w

# Linux Application Checker
# Results Page Module (tests_results.pl)
#
# Copyright (C) 2007-2008 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use Fcntl ':flock';
use Fcntl ':seek';

use Analysis; # utils/Analysis.pm

autoflush_on();

# Reads information about the tested application
sub read_results_info($) {
	my ($res_dir) = @_;

	my $app_name = "";
	if (open(FILE, "$res_dir/INFO")) {
		while (<FILE>) {
			if (m/Application: (.*)$/) {
				$app_name = $1;
				last;
			}
		}
		close(FILE);
	}
	my $res_dir_esc = $res_dir;
	$res_dir_esc =~ s/([ ()])/\\$1/g;
	my $host_platform = `cat $res_dir_esc/test_logs/host_platform`;
	my $lsb_version = `cat $res_dir_esc/test_logs/lsb_version`;
	$host_platform =~ s/[\r\n]//g;
	$lsb_version =~ s/[\r\n]//g;

	return ($app_name, $host_platform, $lsb_version);
}


if ($_POST{"remove"}) {
	if (open(FILE, "+<".$CONFIG{"RESULTS_DIR"}."/HISTORY")) {
		flock(FILE, LOCK_EX);
		while(<FILE>) {
			s/[\r\n]//g;
			next if ($_ eq "");
			my $path = $CONFIG{"RESULTS_DIR"}."/$_";
			if (-d $path) {
				if (defined($_POST{$_})) {
					$path =~ s/([ ()])/\\$1/g;
					system("rm -rf $path");
				}
				else {
					push @list, $_;
				}
			}
		}
		seek(FILE, 0, SEEK_SET);
		truncate(FILE, 0);
		print FILE join("\n", @list)."\n";
		flock(FILE, LOCK_UN);
		close(FILE);
	}
	else {
		print STDERR "Cannot open HISTORY file!\n";
	}
	print "HTTP/1.0 302 Moved\nLocation: tests_results.pl\n\n";
	exit;
}

print "HTTP/1.0 200 OK\n";
print "Content-type: text/html\n\n";

my $popup_html_header = <<HEREDOC;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en" dir="ltr">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />

	<style type="text/css">
		body {
			font: small sans-serif;
		}
		a {
			text-decoration: none;
		}
		a:hover {
			text-decoration: underline;
		}
	</style>
</head>
<body>
HEREDOC

my $popup_html_footer = '</body></html>';

my @res_tab_ids = ('distrs', 'libs', 'ints', 'cert');
my %res_tab_names = (
	'distrs' => 'Distribution Compatibility',
	'libs' => 'Required Libraries',
	'ints' => 'Required Interfaces',
	'cert' => 'LSB Certification'
);

if ($_GET{"details"}) {
	$_GET{'tab'} = 'distrs' if (!$_GET{'tab'});
	my $res_dir = $CONFIG{"RESULTS_DIR"}."/".$_GET{"details"};
	my ($app_name, $host_platform, $lsb_version) = read_results_info($res_dir);
	my $header_info_tabs = '';

	# If we selected one of the correct tabs, print the header.
	if (defined($res_tab_names{$_GET{"tab"}})) {
		my $err = load_analysis_data($res_dir);

		if ($err) {
			# TODO: print a better error message
			print '<p>Error: '.$err.'</p>';
			exit;
		}
		$header_info_tabs = get_title($app_name, $host_platform, $res_dir).
							summary($host_platform, $lsb_version).
							html_results_tabs($_GET{'tab'});
	}

	if ($_GET{"tab"} eq 'distrs') {

		if ( $_GET{"id"} ) {
			my $Did = $_GET{"id"};

			print $popup_html_header;

			print distr_details($Did);

			print $popup_html_footer;

			exit 0;
		}

		# Print header
		print_header("Test Report", "res-distrs");
		print $header_info_tabs;

		print "You application requires <a href=\"tests_results.pl?details=$_GET{'details'}&amp;tab=libs\">".scalar(keys %$Analysis::libs)." external libraries</a>. The table below shows the presence status of these libraries in the distributions analyzed by the Linux Foundation. Your application will run on the \"<font color=\"green\">green</font>\" distributions without loader problems. But please note that functional correctness is in no way guaranteed.\n";

		print D_table( $host_platform, $lsb_version );

		print_footer();
		exit 0;
	}
	elsif ($_GET{"tab"} eq 'libs') {
		if ( $_GET{"lib"} ) {
			my $soname = $_GET{"lib"};

			print $popup_html_header;

			print lib_details($soname, $lsb_version, $host_platform);

			print $popup_html_footer;

			exit 0;
		}

		## POSTed selection:
		# Struct: $distr_sel->{$Did} = $checked;
		my $distr_sel;

		if ( defined $_POST{"submitted"} ) {
			# User's selection
			for my $distr ( values %$Analysis::all_distrs ) {
				next if $distr->{ARCH} ne $host_platform; # arch filter

				my $Did = $distr->{ID};
				my $control_name = "d".$Did;
				if ( $_POST{$control_name} ) {
					$distr_sel->{$Did} = 1;
				} else {
					$distr_sel->{$Did} = 0;
				}
			}
		} else {
			# Default values
			for my $distr ( values %$Analysis::all_distrs ) {
				next if $distr->{ARCH} ne $host_platform; # arch filter
				my $Did = $distr->{ID};
				$distr_sel->{$Did} = 1;
			}
		}

		if ( $_GET{"list"} ) {
		# Library details
			my $soname = $_GET{"list"};

			if ( $_GET{"rel"} ) {
				my $reopen = '?details='.$_GET{'details'}.'&tab=libs&list='.$soname;

				print $popup_html_header;

				print <<HEREDOC;
<script language="javascript" type="text/javascript">
// <![CDATA[
if ( opener && opener.document.hiddensel ) {
	op_form = opener.document.hiddensel;
	op_form.action = '$reopen';
	op_form.target = window.name;
	op_form.submit();
} else {
	document.location = '$reopen';
}
// ]]>
</script>
<noscript>javascript disabled</noscript>
HEREDOC
				print $popup_html_footer;
			}
			else {
				print $popup_html_header;

				print '<h2>Library: <u>'.$soname.'</u></h2>';

				# Presence in Distributions
				print '<h3>Presence in Distributions</h3>';

				print lib_distr_list($soname, $distr_sel);

				print $popup_html_footer;
			}

			exit 0;
		}

		# Print the view
		print_header("Test Report", "res-libs");
		print $header_info_tabs;

		print "The table below lists all external libraries required by your application (based on DT_NEEDED ELF section). There is LSB and Distribution presence status for each library. You can adjust distributions of interest and the report will be updated appropriately.<br /><br />\n";
		print distr_selector( $distr_sel );

		print L_table( $distr_sel, $lsb_version );

		print_footer();
		exit 0;
	}
	elsif ($_GET{"tab"} eq 'ints') {
		my $name_i = "required_ints";
		print_header("Test Report", "res-ints");
		print $header_info_tabs;

		print <<DATA;
The table below lists all external interfaces required by your application. There is LSB status for each interface. Distribution presence analysis will be added in future versions.
<br /><br />

<script language="javascript" type="text/javascript">
// <![CDATA[

var filter = new Array();
filter['ints_table'] = new Array();
filter['ints_table']['lsb'] = 'nlsb';
filter['ints_table']['lib'] = '';

function apply_filter(id, value_lsb, value_lib) {
	var table_name = id + '_table';
	if (value_lsb == "*all*") {
		filter[table_name]['lsb'] = '';
	}
	else if (value_lsb == "ylsb") {
		filter[table_name]['lsb'] = 'ylsb';
	}
	else if (value_lsb == "nlsb") {
		filter[table_name]['lsb'] = 'nlsb';
	}
	if (value_lib == "*all*") {
		filter[table_name]['lib'] = '';
	}
	else if (value_lib != null) {
		filter[table_name]['lib'] = value_lib;
	}

	var table_elem = document.getElementById(table_name);
	var elems = table_elem.getElementsByTagName('TR');
	var num_rows_shown = 0;
	var filter_restriction = '';
	if (filter[table_name]['lsb'] != '') {
		filter_restriction += filter[table_name]['lsb'] + '_' + id;
	}
	if (filter[table_name]['lib'] != '') {
		filter_restriction += ' ' + filter[table_name]['lib'] + '-rule';
	}
	for (var i=0; i<elems.length; ++i) {
		if (!elems[i].className)
			continue;
		if (elems[i].className.indexOf(filter_restriction) != -1) {
			elems[i].style.display = '';
			++num_rows_shown;
		}
		else {
			elems[i].style.display = 'none';
		}
	}

	var elem_empty = document.getElementById(id + '_empty_row');
	var elem_num_shown = document.getElementById(id + '_num_shown');
	if (elem_empty && elem_num_shown) {
		elem_empty.style.display = ((num_rows_shown == 0) ? '' : 'none');
		elem_num_shown.innerHTML = num_rows_shown;
	}
	var elem_empty_text = document.getElementById(id + '_empty_row_text');
	if (elem_empty_text) {
		if (filter[table_name]['lsb'] == 'ylsb')
			elem_empty_text.innerHTML = 'No LSB entries';
		else if (filter[table_name]['lsb'] == 'nlsb')
			elem_empty_text.innerHTML = 'No non-LSB entries';
		else
			elem_empty_text.innerHTML = 'No entries';
	}
	if (filter[table_name]['lib'] != '')
		elem_empty_text.innerHTML += ' in ' + filter[table_name]['lib'];
}

function toggle(id) {
	var elem = document.getElementById(id);
	if (elem) {
		elem.style.display = (elem.style.display == 'none' ? '' : 'none');
	}
	return false;
}
// ]]>
</script>

DATA
		my $buffer = '';
		my $items_num = 0;
		my $nlsb_items_num = 0;
		my $buffer_filter = '';
		my %lib_list = ();
		if (open(FILE_I, "$res_dir/$name_i")) {
			$buffer_filter = <<DATA;
<table class='bartable'>
  <tr>
    <td align="right"><b>Filter by:</b></td>
    <td align="right">
      Presence in LSB:
      <select name="lsb_filter_ints" id="lsb_filter_ints" onchange="javascript:apply_filter('ints', this.value, null);" style="width: 12.5em;">
        <option value="*all*">*No filter*</option>
        <option value="ylsb">Show LSB only</option>
        <option value="nlsb" selected="selected">Show non-LSB only</option>
      </select>
    </td>
    <td align="right">
      Library:
      <select name="lsb_filter_ints_l" id="lsb_filter_ints_l" onchange="javascript:apply_filter('ints', null, this.value);" style="width: 12.5em;">
        <option value="*all*">*No filter*</option>
DATA
			$buffer = <<DATA;
<table class="wikitable" id="ints_table">
  <tr>
    <th style="width: 33em;">Interface</th>
    <th style="width: 6em;">Library</th>
    <th style="width: 4em;">In&nbsp;LSB&nbsp;$lsb_version?</th>
    <th style="width: 33em;">Application&nbsp;Modules</th>
    <th style="width: 25em;">Comments</th>
  </tr>
DATA
			my $row_n = 0;
			while (<FILE_I>) {
				s/[\r\n]//g;
				my @line = split(/;/, $_, 7);
				my ($mgl_name, $disp_name, $version, $in_lsb, $app_modules, $lib_name, $comments) = split(/;/, $_, 7);
				$lib_list{$lib_name} = 1;
				my $tooltip = "";
				my $cont_text = "";
				if ($disp_name eq "") {
					$disp_name = $mgl_name;
					if (length($disp_name) > 50) {
						$tooltip = " title=\"$disp_name\"";
						$cont_text = " <font color=\"red\">&raquo;</font>";
						$disp_name = substr($disp_name, 0, 50)."&hellip;";
					}
				}
				$disp_name =~ s/&/&amp;/g;
				$disp_name =~ s/</&lt;/g;
				$disp_name =~ s/>/&gt;/g;
				my @app_modules_arr = split(/,/, $app_modules);
				$app_modules = '';
				foreach my $component (@app_modules_arr) {
					my $component_disp_path = $component;
					$component =~ s/^.*\.rpm\|([^|]+)/RPM:$1/;
					$component =~ s/^.*\.(tar\.gz|tgz)\|([^|]+)/TGZ:$2/;
					$component =~ s/^PKG:.*\|([^|]+)/PKG:$1/;
					$component =~ s/\|//g;
					$component_disp_path =~ s/\|$//;
					$component_disp_path =~ s/\|/ =&gt; /g;
					$app_modules .= (($app_modules eq '') ? '' : '<br />')."<span title=\"$component_disp_path\">$component</span>";
				}
				++$row_n;
				my $components = scalar(@app_modules_arr);
				$components .= (($components > 1) ? ' components' : ' component');
				$app_modules = "<a href=\"\" onclick=\"javascript:return toggle('int-$row_n');\">Required by $components (&hellip;)</a><br /><div id=\"int-$row_n\" style=\"display: none;\">$app_modules</div>";
				my $lib_name_disp = $lib_name;
				if ($lib_name) {
					$lib_name_disp =~ s/\+/%2B/g;
					$lib_name_disp = "<a href=\"http://www.linuxfoundation.org/navigator/browse/lib_single.php?cmd=list-by-name&amp;Lname=$lib_name_disp\" target=\"_blank\">$lib_name</a>";
				}
				else {
					$lib_name_disp = '<font color="#a0a0a0">&lt;Undefined&gt;</font>';
					$lib_name = '-undef-';
				}
				if ($comments eq "") {
					$comments = "<font color=\"#a0a0a0\">No information</font>";
				}
				if ($in_lsb) {
					$items_num++;
					$buffer .= "  <tr class=\"ylsb_ints $lib_name-rule\" style=\"display: none;\"><td style=\"white-space: normal;\"><a href=\"http://www.linuxfoundation.org/navigator/browse/int_single.php?cmd=list-by-name&amp;Iname=$mgl_name\" target=\"_blank\"$tooltip>$disp_name</a>$cont_text</td><td>$lib_name_disp</td><td align=\"center\"><font color=\"green\"><b>Yes</b></font></td><td>$app_modules</td><td>$comments</td></tr>\n";
				}
				else {
					$items_num++;
					$nlsb_items_num++;
					$buffer .= "  <tr class=\"nlsb_ints $lib_name-rule\"><td style=\"white-space: normal;\"><a href=\"http://www.linuxfoundation.org/navigator/browse/int_single.php?cmd=list-by-name&amp;Iname=$mgl_name\" target=\"_blank\"$tooltip>$disp_name</a>$cont_text</td><td>$lib_name_disp</td><td align=\"center\"><font color=\"red\"><b>No</b></font></td><td>$app_modules</td><td>$comments</td></tr>\n";
				}
			}
			$buffer .= "<tr id=\"ints_empty_row\"".(($nlsb_items_num == 0) ? "" : " style=\"display: none;\"")."><td colspan=\"5\" align=\"center\"><span id=\"ints_empty_row_text\" style=\"color: #a0a0a0\">No non-LSB entries</span></td></tr>\n";
			$buffer .= "</table>\n\n";
			close(FILE_I);
		}
		else {
			$buffer .= "<b><font color=\"red\">Sorry, the report '".$_GET{"details"}."/$name_i' cannot be opened!</font><br />\nReason: $!</b>\n";
		}
		foreach (sort keys %lib_list) {
			if ($_) {
				$buffer_filter .= "        <option value=\"$_\">$_</option>\n";
			}
			else {
				$buffer_filter .= "        <option value=\"-undef-\">&lt;Undefined&gt;</option>\n";
			}
		}
		$buffer_filter .= <<DATA;
      </select>
    </td>
  </tr>
</table>
<br /><b><span id="ints_num_shown">$nlsb_items_num</span></b> of <b>$items_num</b> total entries are shown.
DATA
		print <<DATA;
$buffer_filter$buffer
DATA
	}
	elsif ($_GET{"tab"} eq 'cert') {
		my $name = "test_log.html";
		print_header("Test Report", "res-cert");
		if (open(FILE, $CONFIG{"RESULTS_DIR"}."/".$_GET{"details"}."/$name")) {
			my $transfer = 0;
			while (<FILE>) {
				if (m/<body>/) {
					s/^.*<body>\n?//i;
					$transfer = 1;
				}
				if ($transfer) {
					s/(<\/body>|<\/html>)\n?//i;
					if (m/<!-- Test Report Header -->/) {
						print $header_info_tabs;
						print <<DATA;
<form action="cert_integration.pl?start" method="post" name="integr_data" style="display:none">
<input type="hidden" name="file" value="$CONFIG{'RESULTS_DIR'}/$_GET{'details'}/$_GET{'details'}.tgz" />
</form>
<p style="margin-top: 1.5em;"><a href="cert_integration.pl?start" onclick="javascript:document.integr_data.submit(); return false"><img src="images/medal_small.jpg" width="28" height="28" alt="" /></a> If there are no errors, you can easily <a href="cert_integration.pl?start" onclick="document.integr_data.submit(); return false"><b>apply for certification</b></a> by uploading the results to the <a href="http://$cert_sys_host$cert_sys_base" target="_blank">LSB Certification System</a>.</p>
DATA
					}
					else {
						print $_;
					}
				}
			}
			close(FILE);
		}
		else {
			print "<b><font color=\"red\">Sorry, the report '".$_GET{"details"}."' cannot be opened!</font><br />\nReason: $!</b>\n";
		}
	}
	else {
		print_header("Test Report", "res-distrs");
		print "<b><font color=\"red\">Invalid tab selected!</font></b>\n";
	}
}
else {
	print_header("Test Results", "results");

	print "<h1>Test Results</h1>\n";

	print "<form method=\"post\" action=\"tests_results.pl\" name=\"results_list\">\n<table class=\"main\">\n";
	print "  <tr>\n    <td><input type=\"checkbox\" name=\"check_all\" id=\"check_all\" onclick=\"javascript:check_unckeck_all();\" /></td>\n";
	print "    <th style=\"width: 14em;\">Date/Time</th>\n";
	print "    <th style=\"width: 14em;\">Summary</th>\n";
	print "    <th style=\"width: 25em;\">Comment</th>\n  </tr>\n";
	my @list = ();
	if (open(FILE, "+<".$CONFIG{"RESULTS_DIR"}."/HISTORY")) {
		flock(FILE, LOCK_EX);
		while(<FILE>) {
			s/[\r\n]//g;
			next if ($_ eq "");
			push @list, $_ if (-d $CONFIG{"RESULTS_DIR"}."/$_");
		}
		seek(FILE, 0, SEEK_SET);
		truncate(FILE, 0);
		print FILE join("\n", @list)."\n";
		flock(FILE, LOCK_UN);
		close(FILE);
		@list = reverse(@list);
		foreach my $name (@list) {
			$_ = $CONFIG{"RESULTS_DIR"}."/$name";
			my @summary = ();
			my @comment = ();
			my $current_field = \@summary;
			if (open(FILE, "$_/INFO")) {
				while (<FILE>) {
					s/&/&amp;/g;
					s/</&lt;/g;
					s/>/&gt;/g;
					s/[\r\n]//g;
					if (($_ eq "") and ($current_field != \@comment)) {
						$current_field = \@comment;
						next;
					}
					push @$current_field, $_;
				}
				close(FILE);
			}
			my $fail = -1;
			my $app_name = "";
			my @extraction = ();
			foreach (@summary) {
				if (m/^Verdict: (\S+)/i) {
					if ($1 eq "Success") {
						$fail = 0;
					}
					elsif ($1 eq "Warnings") {
						$fail = 0;
					}
					elsif ($1 eq "Failed") {
						$fail = 1;
					}
					elsif ($1 eq "No result") {
						$fail = 2;
					}
				}
				elsif (m/^Application: (.*)$/i) {
					$app_name = $1;
				}
				else {
					# Align numbers by right side (6 digits max)
					s/^(\d+) /("&nbsp;"x(6-length($1))).$1." "/e;
					push @extraction, $_;
				}
			}
			my $jsummary = join("<br />", @extraction);
			$jsummary = "&nbsp;" if ($jsummary eq "");
			my $jcomment = join("<br />", @comment);
			if ($jcomment ne "") {
				$jcomment =~ s/&gt;/>/g;
				$jcomment =~ s/&lt;/</g;
				$jcomment =~ s/&amp;/&/g;
			}
			else {
				$jcomment = "&nbsp;";
			}
			my $image = "";
			my $img_alt = "";
			if ($fail == -1) {
				$fail = ($jsummary =~ m/(\d+) Tests Failed/) ? ($1 ? 1 : 0) : -1;
			}
			if ($fail) {
				$image = "TestsFailed.png";
				$img_alt = "TestsFailed";
			}
			else {
				$image = "TestsPassed.png";
				$img_alt = "TestsPassed";
			}
			my $show_name = $name;
			$show_name =~ s/^.*-([^\-]*)\-([^\-]*)\-([^\-]*)\-([^\-]*)h\-([^\-]*)m\-([^\-]*)s/$3.$2.$1&nbsp;$4:$5:$6/;
			my $upload_icon = "";
			if (! -f $CONFIG{"RESULTS_DIR"}."/$name/test_logs/ftp_upload") {
				$upload_icon = "&nbsp;<a href=\"upload_data.pl?details=$name\" title=\"Upload Data to the Linux Foundation\"><img src=\"images/upload_data.png\" alt=\"\" width=\"16\" height=\"16\" /></a>";
			}
			print <<DATA;
  <tr>
    <td rowspan="3"><input type='checkbox' name='$name' onclick='javascript:update_state();' /></td>
    <td rowspan="1" style="border-bottom: none;"><img src="images/environment-spacer.gif" width="1" height="16" alt="" /></td>
    <td rowspan="3" valign="top"><code>$jsummary</code></td>
    <td rowspan="3" valign="top">$jcomment</td>
  </tr>
  <tr>
    <td valign="middle" style="border: none;">
      <table cellpadding="0" cellspacing="0" border="0"><tr>
        <td style="border: none; padding: 0px;"><img src="images/$image" alt="$img_alt" width="33" height="30" /></td>
        <td style="border: none; padding: 0px;">&nbsp;<a href="tests_results.pl?details=$name">$show_name</a><br />&nbsp;Application: $app_name</td>
      </tr></table>
    </td>
  </tr>
  <tr>
    <td style="border: none; text-align: right; vertical-align: bottom;" width="5" height="5">
      <a href="tests_results.pl?details=$name" title="Show detailed report"><img src="images/report_details.gif" alt="" width="16" height="16" /></a>&nbsp;<a href="tests_results.pl?details=$name&amp;tab=ints" title="Show analysis results"><img src="images/report_summary.gif" alt="" width="16" height="16" /></a>$upload_icon
    </td>
  </tr>
DATA
		}
	}
	print <<DATA;
</table>
<br /><input type="submit" name="remove" id="remove_button" value="Remove Selected Entries" disabled="disabled" onclick='javascript:return confirm_remove();' />
</form>

<script language="javascript" type="text/javascript">
// <![CDATA[
function count_checked() {
	var num = 0;
	var form = document.results_list;
	for (var i=0; i<form.length; ++i) {
		if ((form[i].type.toLowerCase() == "checkbox") && (form[i].name != "check_all") && form[i].checked) {
			++num;
		}
	}
	return num;
}

function update_state() {
	var elem = document.getElementById("remove_button");
	if (elem) {
		var num = count_checked();
		elem.disabled = (num == 0);
	}
}

function check_unckeck(box, check) {
	temp = box.onchange;
	box.onchange = null;
	box.checked = check;
	box.onchange = temp;
}

function check_unckeck_all() {
	var elem = document.getElementById("check_all");
	if (elem) {
		var checked = elem.checked;
		var form = document.results_list;
		for (var i=0; i<form.length; ++i) {
			if ((form[i].type.toLowerCase() == "checkbox") && (form[i].name != "check_all"))
				check_unckeck(form[i], checked);
		}
		update_state();
	}
}

function confirm_remove() {
	var num = count_checked();
	if (num == 0) {
		alert("Nothing selected!");
		return false
	}
	else
		return confirm("Do you wish to delete " + num + " selected report(s)?");
}
// ]]>
</script>
DATA
}

print_footer();

#-----------------------------------------------------------------------

# Uses Analysis module to load all the analysis results.
sub load_analysis_data {
	my ($res_dir) = @_;

	# Load distribution list (contains distribution name, version, database ID, architecture)
	Analysis::load_distrs($res_dir."/Distributions")
		or return "Failed to read the Distributions file.";

	# First, check the cache files
	my $libs_file      = $res_dir."/libs";
	my $lib_attrs_file = $res_dir."/lib_attrs";
	my $lib_kb_file    = $res_dir."/lib_kb";

	Analysis::load_lib_dump($libs_file, $lib_attrs_file, $lib_kb_file);

	return;
}

# Customizable tab bar
sub html_results_tabs {
	my ($selected) = @_;

	my $nl = "\n";
	my $res = "";

	# Table start
	$res .= '<table cellpadding="0" cellspacing="0" border="0" width="100%"'
		.' style="white-space: nowrap; font-weight: bold; text-align: center; font-size: 120%; margin-bottom: 5pt;">'.$nl;
	$res .= '<tr>'.$nl;

	# Left spacer
	$res .= '<td style="width: 50px; border-bottom: solid 1px #166a98; height: 2em;">';
	$res .= '<img src="images/environment-spacer.gif" width="50" height="1" alt="" />';
	$res .= '</td>'.$nl;

	my $tabs_num = scalar(@res_tab_ids);

	foreach my $id (@res_tab_ids) {
		my $name = $res_tab_names{$id};
		my $link = 'tests_results.pl?details='.$_GET{'details'}.'&amp;tab='.$id;

		# Tab
		if ( $id eq $selected ) {
			$res .= '<td style="width: 15em; border: solid 1px #166a98;'
						.' border-bottom: none; background-color: #a0d5f1; color: #004060;">';
		} else {
			$res .= '<td class="res-tab-highlight" style="width: 15em; border: solid 1px #166a98;"'
				.' onclick="javascript:document.location=\''.$link.'\'">';
		}
		$res .= '<img src="images/environment-spacer.gif" width="10" height="1" alt="" />';
		$name =~ s/\s/&nbsp;/g;
		$res .= $name;
		$res .= '<img src="images/environment-spacer.gif" width="10" height="1" alt="" />';
		$res .= '</td>';

		# Spacer
		$res .= '<td style="width: 10px; border-bottom: solid 1px #166a98;">';
		$res .= '<img src="images/environment-spacer.gif" width="10" height="1" alt="" />';
		$res .= '</td>'.$nl;
	}

	$res .= '<td style="border-bottom: solid 1px #166a98;">';
	$res .= '<img src="images/environment-spacer.gif" width="1" height="1" alt="" />';
	$res .= '</td>';

	$res .= '</tr>';

	# Bottom line
	$res .= '<tr>';
	$res .= '<td colspan="'.(2 * $tabs_num+2).'" style="background-color: #a0d5f1; height: 10px;">';
	$res .= '<img src="images/environment-spacer.gif" width="1" height="1" alt="" />';
	$res .= '</td>';
	$res .= '</tr>';

	$res .= '</table>'.$nl;

	return $res;
}

# Small function for easy formatting a distro name.
sub distr_name {
	my ($distr) = @_;

	return $distr->{NAME}.' '.$distr->{VER};
}

# Renders a control for selecting distribution subset
sub distr_selector {
	my ($distr_sel) = @_;

	my $nl = "\n";
	my $res = "";

	my %distr_names = ();

	$distr_names{ distr_name($Analysis::all_distrs->{$_}) } = $_ for keys %$distr_sel;
	my $sel_distrs = scalar(grep {$_} values %$distr_sel);
	my $total_distrs = scalar(keys %$distr_sel);

	$res .= <<HEREDOC;
<script type="text/javascript" language="javascript">
// <![CDATA[
var num_distrs = $sel_distrs;

function my_reset()
{
	e = document.getElementById('distrsel');

	var inputs = document.getElementsByTagName('input');
	for (var i = 0; i < inputs.length; i++) {
		if ( inputs[i].type == 'checkbox' ) {
			inputs[i].checked = 'checked';
		}
	}
	num_distrs = $total_distrs;
	document.getElementById('sel-distrs').innerHTML = num_distrs;
}

function onDistrSelect(elem) {
	if (elem.checked)
		++num_distrs;
	else
		--num_distrs;
	document.getElementById('sel-distrs').innerHTML = num_distrs;
}
// ]]>
</script>
HEREDOC

	# Toogler
	$res .= '<a href="" onclick="return toggle(\'dseldiv\')"><b><u>Select distribution subset&hellip;</u></b></a> (<b><span id="sel-distrs">'.$sel_distrs.'</span></b> of <b>'.$total_distrs.'</b> selected)';

	# Print selector
	$res .= '<div id="dseldiv" style="display: none;">';
	$res .= '<form name="distrsel" id="distrsel" method="post" action="">'.$nl;
	$res .= '<div style="border:solid 1px;padding:3pt;margin-bottom:3pt;height:200pt;width:300pt;overflow:auto">';
	foreach my $distr_name ( sort {lc $a cmp lc $b} keys %distr_names ) {
		my $Did = $distr_names{$distr_name};

		my $checked = $distr_sel->{$Did} ? 'checked="checked" ' : '';
		my $control_name = "d".$Did;

		$res .= '<input type="checkbox" name="'.$control_name.'" id="'.$control_name.'" '.$checked.' onclick="javascript:onDistrSelect(this);" /> ';
		$res .= $distr_name;
		$res .= '<br />'.$nl;
	}
	$res .= '</div>'.$nl; # border
	$res .= '<input type="submit" name="submitted" value="Apply" style="width: 7em;" />'.$nl;
	$res .= ' &nbsp; &nbsp; ';
	$res .= '<input type="button" value="Reset" style="width: 7em;" onclick="return my_reset()" />'.$nl;
	$res .= '</form>'.$nl;
	$res .= '</div>'.$nl; # hidden

	# Print hidden form with selected distros
	$res .= '<form name="hiddensel" action="" method="post" style="display:none">'.$nl;
	foreach my $Did ( keys %$distr_sel ) {
		next if not $distr_sel->{$Did};

		my $control_name = "d".$Did;

		$res .= '<input type="hidden" name="'.$control_name.'" value="1" />'.$nl;
	}
	$res .= '<input type="hidden" name="submitted" value="1" />'.$nl;
	$res .= '</form>'.$nl;

	return $res;
}

# Library compatibility table
sub L_table {
	my ($distr_sel, $selected_lsb_ver) = @_;

	my $res = "";

	my $nl = "\n"; # Set this to "" to make spaghetti HTML code

	my $link = '?details='.$_GET{'details'}.'&tab=libs';

	$res .= <<HEREDOC;
<script type="text/javascript" language="javascript">
// <![CDATA[
function toggle( id )
{
	var v = document.getElementById( id );
	if ( v ) {
		v.style.display = (v.style.display == 'none' ? '' : 'none');
	}
	return false;
}

function popup_win( link )
{
	var w = window.open( link, 'details',
			'toolbar=0,menubar=0,scrollbars=1,height=750,width=850,dependent=yes,dialog=yes'
		);
	w.moveTo(100,0);
	w.focus();

	return w;
}

function det_open( soname )
{
	popup_win( '$link' + '&lib=' + soname );

	return false;
}

function lib_distrs_open( soname )
{
	popup_win( '$link' + '&rel=1&list=' + soname );

	return false;
}
// ]]>
</script>
HEREDOC

	my $filtered = scalar grep {!$_} values %$distr_sel;

	# Table Header
	$res .= '<table id="l_table" class="wikitable" width="100%">'.$nl;
	$res .= '<tr>'
		.'<th>Library</th>'
		.'<th>Dependent<br />Application Components</th>'
		.'<th>Interfaces<br />Actually Used</th>'
		.'<th>Presence in<br />'.($filtered ? 'Selected ' : '').'Distributions</th>'
		.'<th id="lsb_col_header">In LSB '.$selected_lsb_ver.'?</th>'
		.'<th>More Info</th>'
		.'</tr>'.$nl;

	my $libs = $Analysis::libs;

	my $row_n = 0; # Counter for generating row-specific IDs

	# Table Body
	foreach my $soname ( sort {lc $a cmp lc $b} keys %$libs ) {
		my $lib_obj = $libs->{$soname};

		$row_n++;

		$res .= '<tr>'.$nl;

		## Library
		my $soname_link = $soname;
		$soname_link =~ s/\+/%2B/g;
		$res .= '<td><b><a href="http://www.linuxfoundation.org/navigator/browse/rawlib.php?cmd=list-byrunname&amp;RLrunname='.$soname_link.'" target="_blank">'.$soname.'</a></b></td>'.$nl;

		# App Modules: preparation
		my $app_modules = "";
		if ( $lib_obj->{APP_MODULES} ) {
			foreach my $component ( @{$lib_obj->{APP_MODULES}} ) {
				my $component_disp_path = $component;
				$component =~ s/^.*\.rpm\|([^|]+)/RPM:$1/;
				$component =~ s/^.*\.(tar\.gz|tgz)\|([^|]+)/TGZ:$2/;
				$component =~ s/^PKG:.*\|([^|]+)/PKG:$1/;
				$component =~ s/\|//g;
				$component_disp_path =~ s/\|$//;
				$component_disp_path =~ s/\|/ =&gt; /g;
				$app_modules .= (($app_modules eq '') ? '' : '<br />')."<span title=\"$component_disp_path\">$component</span>";
			}
		}


		## App modules expandable list
		my $div_id = "cl".$row_n;

		$res .= '<td>';
		my $plu = ( @{$lib_obj->{APP_MODULES}} > 1 ? 's' : '' );
		$res .= '<a href="" onclick="return toggle(\''.$div_id.'\')">'
					.'Required by '.scalar(@{$lib_obj->{APP_MODULES}}).' component'.$plu.' (&hellip;)'
				.'</a>';
		$res .= '<div id="'.$div_id.'" style="display:none">';
		$res .= $app_modules;
		$res .= '</div>';
		$res .= '</td>'.$nl;

		# Interfaces used
		my $ints_used = ($lib_obj->{INTS_USED} or 0);
		$res .= '<td align="center">';
		if ( $ints_used ) {
			$res .= $ints_used
		} else {
			$res .= '<span style="color:red"><b>'.$ints_used.'</b></span>';
		}
		$res .= '</td>'.$nl;

		## Presence in distributions

		my @selected_distrs = grep { $distr_sel->{$_} } keys %$distr_sel;

		my $total = scalar( @selected_distrs );
		my $present = 0;
		if ( $lib_obj->{DISTRS} ) {
			$present = scalar( grep { $lib_obj->{DISTRS}{$_} } @selected_distrs );
		}
		my $approved = ($lib_obj->{APPROVED} or 0);

		my $bgcolor;
		if ( $present == $total ) {
			$bgcolor = "#CCFFCC"; # red
		} elsif ( $present > 0 ) {
			$bgcolor = "#FEFECC"; # yellow
		} else {
			$bgcolor = "#FFCCCC"; # green
		}

		$res .= '<td align="right" style="background:'.$bgcolor.'">';
		if ( $approved or $present == $total ) {
			$res .= $present." of ".$total;
		} elsif ( $present > 0 ) {
			$res .= $present." known distro".($present > 1 ? 's' : '');
		} else {
			$res .= "Unknown library";
		}

		$res .= ' (<a href="'.$link.'&list='.$soname.'"'
					.' onclick="return lib_distrs_open(\''.$soname.'\')">details&hellip;</a>)';
		$res .= '</td>'.$nl;


		## LSB
		$res .= '<td align="center">';

		$res .= '<div>';

		my ($lsb_status, $lsb_status_ver) = lib_lsb_status($lib_obj, $selected_lsb_ver);

		if ( !defined $lsb_status ) { die; }

		my $color = undef;

		if ( $lsb_status eq "NOT" ) {
			$res .= '<span style="color:red">No</span>';
		}
		elsif ( $lsb_status eq "CANDIDATE" ) {
			$res .= '<span style="color:blue">Candidate for '.$lsb_status_ver.'</span>';
		}
		elsif ( $lsb_status eq "OPTIONAL" ) {
			$res .= '<span style="color:blue">Optional</span>';
		}
		elsif ( $lsb_status eq "IN" ) {
			$res .= '<span style="color:green">Yes</span>';
		}
		elsif ( $lsb_status eq "DEPRECATED" ) {
			$res .= '<span style="color:blue">Deprecated</span>';
		}

		$res .= '</div>';

		$res .= '</td>'.$nl;


		## More Info

		$res .= '<td>';
		my $info_link = $link.'&rel=1&lib='.$soname;
		my $info_onclick = 'onclick="return det_open(\''.$soname.'\')"';
		if ( $lib_obj->{ATTR} || $Analysis::libKBDistr->{$soname} ) {
			$res .= '<a href="'.$info_link.'" '.$info_onclick.'><b>Known Issues&hellip;</b></a>';
		} else {
			$res .= '<a href="'.$info_link.'" '.$info_onclick.'>Details&hellip;</a>';
		}
		$res .= '</td>'.$nl;

		$res .= '</tr>'.$nl;
	}

	# End of the table
	$res .= '</table>'.$nl;

	return $res;
}

# Returns LSB status of the library and LSB version related to this status
sub lib_lsb_status {
	my ($lib, $lsb_ver) = @_;

	# TODO: search by libname if nothing was found by soname (in caller). Maybe another version of this lib is in LSB.

	return ("NOT") if !$lib;

	my $in = 0;
	if (  $lib->{APPEAREDIN} && $lib->{APPEAREDIN} le $lsb_ver ) {
		if ( ! $lib->{WITHDRAWNIN} || $lib->{WITHDRAWNIN} gt $lsb_ver ) {
			$in = 1;
		}
	}

	if ( not $in ) {
		if ( $lib->{APPEAREDIN} && $lib->{APPEAREDIN} gt $lsb_ver ) {
			return ("NOT", $lib->{APPEAREDIN});
		} elsif ( $lib->{CANDIDATEFOR} ) {
			return ("CANDIDATE", $lib->{CANDIDATEFOR});
		} else {
			return "NOT";
		}
	} else {
		if ( $lib->{DEPRECATEDSINCE} && $lib->{DEPRECATEDSINCE} le $lsb_ver ) {
			return ("DEPRECATED", $lib->{DEPRECATEDSINCE});
		} else {
			if ( $lib->{MANDATORYSINCE} && $lib->{MANDATORYSINCE} le $lsb_ver ) {
				return ("IN", $lib->{MANDATORYSINCE});
			} else {
				return ("OPTIONAL", $lib->{APPEAREDIN});
			}
		}
	}
}

# Contents of a popup window with details on a library presence
# in a selected distribution subset.
sub lib_distr_list {
	my ($soname, $distr_sel) = @_;

	my $nl = "\n";
	my $res = "";

	my $lib = $Analysis::libs->{$soname};

	my $filtered = 0;

	my %present = ();
	my %missed = ();
	foreach my $Did ( keys %$distr_sel ) {
		if ( not $distr_sel->{$Did} ) {
			$filtered = 1;
			next; # Skip unselected
		}

		my $distr_name = distr_name( $Analysis::all_distrs->{$Did} );
		if ( $lib->{DISTRS}{$Did} ) {
			$present{$distr_name} = $Did;
		} else {
			$missed{$distr_name} = $Did;
		}
	}

	my $approved = ($lib->{APPROVED} or 0);

	my $present_num = scalar(keys %present);
	my $missed_num = scalar(keys %missed);
	if ( $present_num > 0 ) {
		$res .= '<p>';
		$res .= 'Present in <span style="color:green">'.$present_num.'</span>';
		if ( $approved or $missed_num == 0) {
			$res .= ' and missed in <span style="color:red">'.$missed_num.'</span>'
		} else {
			$res .= ' and nothing is known about <span style="color:#907000">'.$missed_num.'</span>'; # yellow
		}
		if ( $filtered ) {
			$res .= ' of '.($present_num + $missed_num).' selected distributions.';
		} else {
			$res .= ' of total '.($present_num + $missed_num).' distributions.';
		}
		$res .= '</p>'.$nl;
	} else {
		if ( $approved ) {
			$res .= '<p>This library is missed in all the '.($present_num + $missed_num);
			if ( $filtered ) {
				$res .= ' selected distributions.</p>'.$nl;
			} else {
				$res .= ' analyzed distributions.</p>'.$nl;
			}
		} else {
			$res .= '<p>There is no information about this library in the database.</p>'.$nl;
		}
	}

	# TODO: single green/red list?

	local *distr_list = sub {
		my ($color, $hash) = @_;

		my $res = "";

		$res .= '<ul>'.$nl; # Green list
		for my $distr_name ( sort {lc $a cmp lc $b} keys %$hash ) {
			my $Did = $hash->{$distr_name};
			my $link = '?details='.$_GET{'details'}.'&tab=distrs&id='.$Did;
			$link =~ s/&/&amp;/g;
			$res .= '<li><a style="color:'.$color.'" href="'.$link.'">'.$distr_name.'</a></li>'.$nl;
		}
		$res .= '</ul>'.$nl;

		return $res;
	};

	if ( %present ) {
		$res .= '<h4>Present in:</h4>'.$nl;

		$res .= distr_list('green', \%present);
	}

	if ( %missed ) {
		if ( $approved ) {
			$res .= '<h4>Missed in:</h4>'.$nl;

			$res .= distr_list('red', \%missed);
		} else {
			$res .= '<h4>No information for:</h4>'.$nl;

			$res .= distr_list('#907000', \%missed);
		}
	}

	return $res;
}

# Contents of a details popup window for Libraries table
sub lib_details {
	my ($soname, $selected_lsb_ver, $architecture) = @_;

	my $nl = "\n";
	my $res = "";

	my $lib = $Analysis::libs->{$soname};

	$res .= '<h2>Library: <u>'.$soname.'</u></h2>';

	$res .= '<hr/>'.$nl;

	# Check Lib Knowledge Base
	my $kb_info = lib_KB_info($soname);
	if ( $kb_info ) {
		$res .= '<h3>Known compatibility issues:</h3>'.$nl;
		$res .= '<p>'.$kb_info.'</p>'.$nl;

		$res .= '<hr />'.$nl;
	}

	# Check LibAttr
	my $attr = "";
	if ( $lib->{ATTR} ) {
		foreach my $type ( sort keys %{$lib->{ATTR}} ) {
			if ( $type eq "Recommendation" ) {
				$attr .= " Recomendation: ".$lib->{ATTR}->{$type};
				$attr .= "." if $attr =~ /[^\s\.]\s*$/;
			}
			elsif ( $type eq "StaticLinkage" ) {
				$attr .= " May be linked statically. ".$lib->{ATTR}->{$type};
				$attr .= "." if $attr =~ /[^\s\.]\s*$/;
			}
			elsif ( $type eq "Rejection" ) {
				$attr .= " ".$lib->{ATTR}->{$type};
				$attr .= "." if $attr !~ /[^\s\.]\s*$/;
			}
			else { # Custom
				$attr .= " ".$lib->{ATTR}->{$type};
			}
		}
	}
	$attr =~ s/^\s+//; # remove unnecessary spaces
	$attr =~ s/\s+$//; #

	if ( $attr ) {
		$res .= '<h3>More info</h3>'.$nl;
		$res .= '<p>'.$attr.'</p>'.$nl;

		$res .= '<hr />'.$nl;
	}

	# LSB Status
	$res .= '<h3>LSB Status:</h3>'.$nl;

	my ($lsb_status, $lsb_status_ver) = lib_lsb_status($lib, $selected_lsb_ver);

	if ( !$lib ) {
		$res .= "Error: there is no such library with name '".$soname."'";
		return $res;
	}

	if ( $lsb_status eq "NOT" ) {
		$res .= '<div style="color:red">Never been in LSB.</div>'.$nl;
	}
	elsif ( $lsb_status eq "CANDIDATE" ) {
		$res .= '<div style="color:red">Not in LSB. Candidate for LSB '.$lsb_status_ver.'</div>'.$nl;
	}
	elsif ( $lsb_status eq "OPTIONAL" ) {
		$res .= '<div style="color:blue">Optional since LSB '.$lsb_status_ver.'</div>'.$nl;
	}
	elsif ( $lsb_status eq "IN" ) {
		$res .= '<div style="color:green">Mandatory since LSB '.$lsb_status_ver.'</div>'.$nl;
	}
	elsif ( $lsb_status eq "DEPRECATED" ) {
		$res .= '<div style="color:blue">Deprecated since LSB '.$lsb_status_ver.'</div>'.$nl;
	}
	elsif ( $lsb_status eq "WITHDRAWN" ) {
		$res .= '<div style="color:red">Withdrawn since LSB '.$lsb_status_ver.'</div>'.$nl;
	}
	else {
		#$res .= '??? '.$lsb_status.' ???'; # DBG:
	}

	$res .= '<hr />'.$nl;

	# Presence in Distributions
	$res .= '<h3>Presence in Distributions</h3>'.$nl;

	my $distrs = {};
	$distrs->{$_} = 1 for grep {$Analysis::all_distrs->{$_}{ARCH} eq $architecture} keys %$Analysis::all_distrs;

	$res .= lib_distr_list($soname, $distrs);

	$res .= '<hr />'.$nl;

	return $res;
}

# Returns information from the Knowledge Base for specified library and
#  distribution set.
sub lib_KB_info {
	my ($soname) = @_;

	my $nl = "\n";
	my $res = "";

	my $problems = $Analysis::libKBDistr->{$soname};

	return undef if not $problems;

	foreach my $LKBid ( sort keys %$problems ) {
		my $text = $Analysis::libKB->[$LKBid];
		$text =~ s/\n/<br\/>\n/g;
		$text =~ s/&/&amp;/g; # TODO: html() func
		$text =~ s/</&lt;/g;
		$text =~ s/>/&gt;/g;

		$res .= '<div>';
		$res .= "<p>";
		$res .= $text;
		$res .= "</p>".$nl;

		my $grouped = $problems->{$LKBid};

		foreach my $i ( sort keys %$grouped ) {
			$res .= '<div><b>Group '.$i.':</b> ';

			# Print IDs of distributions from this group
			$res .= join ", ", map {distr_name($Analysis::all_distrs->{$_}) } @{$grouped->{$i}};

			$res .= '.</div>'.$nl;
		}

		$res .= '</div>'
	}

	return $res;
}

# Distribution compatibility table
sub D_table {
	my ($architecture, $selected_lsb_ver) = @_;

	my $all_distrs = $Analysis::all_distrs;
	my $libs = $Analysis::libs;

	my @a_distrs = grep {$all_distrs->{$_}{ARCH} eq $architecture} keys %$all_distrs;

	my %by_name = ();
	$by_name{ distr_name($all_distrs->{$_}) } = $_ for @a_distrs;

	my %missed = ();
	my %unknown = ();
	foreach my $Did ( @a_distrs ) {
		foreach my $soname ( keys %$libs ) {
			if ( not exists $libs->{$soname}{DISTRS}{$Did} ) {
				if ( $libs->{$soname}{APPROVED} ) {
					$missed{$Did}{$soname} = 1;
				} else {
					$unknown{$Did}{$soname} = 1;
				}
			}
		}
	}

	my $nl = "\n";
	my $res = "";

	my $link = '?details='.$_GET{'details'}.'&tab=distrs';
	$res .= <<HEREDOC;
<script type="text/javascript" language="javascript">
// <![CDATA[

function popup_win( link )
{
	var w = window.open( link, 'details',
			'toolbar=0,menubar=0,scrollbars=1,height=750,width=850,dependent=yes,dialog=yes'
		);
	w.moveTo(100,0);
	w.focus();

	return w;
}

function det_open( id )
{
	popup_win( '$link' + '&id=' + id );

	return false;
}
// ]]>
</script>
HEREDOC

	$res .= '<table class="wikitable" id="d_table">'.$nl;
	$res .= '<tr>'
		.'<th>Distribution</th>'
		.'<th>Missed&nbsp;Libs</th>'
		.'</tr>'.$nl;

	my $libs_num = scalar( keys %$libs );

	# Distributions list
	foreach my $Dname ( sort {
						 keys %{$missed{$by_name{$a}}} <=> keys %{$missed{$by_name{$b}}}
						 or keys %{$unknown{$by_name{$a}}} <=> keys %{$unknown{$by_name{$b}}}
						 or (lc $a cmp lc $b)
						} keys %by_name )
	{
		my $Did = $by_name{$Dname};

		my $missed_num = 0;
		$missed_num = scalar( keys %{$missed{$Did}} ) if $missed{$Did};
		my $unknown_num = 0;
		$unknown_num = scalar( keys %{$unknown{$Did}} ) if $unknown{$Did};

		my $bgcolor;
		if ( $missed_num ) {
			$bgcolor = "#FFDDDD"; # red
		} elsif ( $unknown_num ) {
			$bgcolor = "#FEFECC"; # yellow
		} else {
			$bgcolor = "#CCFFCC"; # green
		}

		$res .= '<tr id="dtr_'.$Did.'" style="background:'.$bgcolor.'">'.$nl;

		# Distribution
		my $Dname_link = $Dname;
		$Dname_link =~ s/^(.*)? \d.*$/$1/;
		$Dname_link =~ s/ /%20/g;
		$res .= '<td><b><a href="http://www.linuxfoundation.org/navigator/browse/distr.php?cmd=list-byname&Dname='.$Dname_link.'" target="_blank">'.$Dname.'</a></b></td>'.$nl;

		# Missed
		if ( $missed_num or $unknown_num ) {
			$res .= '<td align="right">';
			if ( $missed_num and $unknown_num ) {
				$res .= '<b>'.$missed_num.'</b> missed'
						.' and <b>'.$unknown_num.'</b> unknown lib'.($unknown_num > 1 ? 's' : '').'.';
			}
			elsif ( $missed_num ) {
				$res .= '<b>'.$missed_num.'</b> missed lib'.($missed_num > 1 ? 's' : '').'.';
			}
			else {
				$res .= '<b>'.$unknown_num.'</b> unknown lib'.($unknown_num > 1 ? 's' : '').'.';
			}
			$res .= ' (<a href="" onclick="return det_open('.$Did.')">list&hellip;</a>)';
			$res .= '</td>'.$nl;
		} else {
			$res .= '<td align="center">'
				.'none'
				.'</td>'.$nl;
		}

		$res .= '</tr>'.$nl;
	}

	$res .= '</table>'.$nl;

	return $res;
}

# Contents of a detail popup window for Distributions table
sub distr_details {
	my ($Did) = @_;

	my $nl = "\n";
	my $res = "";

	my $distr = $Analysis::all_distrs->{$Did};

	if ( !$distr ) {
		$res .= "There is no distribution with ID '$Did'.";
		return $res;
	}

	my $libs = $Analysis::libs;
	my %missed = ();
	my %unknown = ();
	my %present = ();
	foreach my $soname ( keys %$libs ) {
		if ( exists $libs->{$soname}{DISTRS}{$Did} ) {
			$present{$soname} = 1;
		} else {
			if ( $libs->{$soname}{APPROVED} ) {
				$missed{$soname} = 1;
			} else {
				$unknown{$soname} = 1;
			}
		}
	}

	my $Dname = distr_name($distr);
	my $Dname_link = $Dname;
	$Dname_link =~ s/^(.*)? \d.*$/$1/;
	$Dname_link =~ s/ /%20/g;
	$res .= '<h2>Distribution: <a href="http://www.linuxfoundation.org/navigator/browse/distr.php?cmd=list-byname&Dname='.$Dname_link.'" target="_blank"><u>'.$Dname.'</u></a></h2>'.$nl;

	local *lib_list = sub {
		my ($color, $hash) = @_;

		my $res = "";

		$res .= '<ul>'.$nl;
		foreach my $soname ( sort {lc $a cmp lc $b} keys %$hash ) {
			my $link = '?details='.$_GET{'details'}.'&tab=libs&lib='.$soname;
			$res .= '<li><a style="color:'.$color.'" href="'.$link.'">'.$soname.'</a></li>'.$nl;
		}
		$res .= '</ul>'.$nl;

		return $res;
	};

	if ( %missed ) {
		$res .= '<h3>Missed Libraries:</h3>'.$nl;

		$res .= lib_list('red', \%missed);
	}

	if ( %unknown ) {
		$res .= '<h3>No Information About:</h3>'.$nl;

		$res .= lib_list('#907000', \%unknown);
	}

	if ( %present ) {
		$res .= '<h3>Present Libraries:</h3>'.$nl;

		$res .= lib_list('green', \%present);
	}

	return $res;
}

# Renders main title of the results page.
sub get_title {
	my ($app_name, $architecture, $res_dir) = @_;

	my $res = "<h1>Analysis Results for <font color=\"blue\">$app_name</font> on <font color=\"blue\">$architecture</font></h1>\n\n";

	if (! -f "$res_dir/test_logs/ftp_upload") {
		$res .= <<DATA;
<div class="upload-suggest" onclick="javascript:document.location='upload_data.pl?details=$_GET{'details'}';">
  <table width="100%" border="0" cellpadding="0" cellspacing="0">
  	<tr>
      <td valign="top"><img src="images/upload_data_l.png" width="28" height="28" alt="" /></td>
      <td><img src="images/environment-spacer.png" width="3" height="1" alt="" /></td>
      <td>Please <a href="upload_data.pl?details=$_GET{'details'}" onclick="javascript:event.cancelBubble=true;"><b>upload</b></a> information about your application to the Registered Applications section of the <a href="http://www.linuxfoundation.org/navigator/browse/app.php" target="_blank" onclick="javascript:event.cancelBubble=true;">LSB Navigator</a>.</td>
    </tr>
  </table>
</div>
DATA
	}
	return $res;
}

# Renders a short summary for the header of the results page.
sub summary {
	my ($architecture, $selected_lsb_ver) = @_;

	my $libs = $Analysis::libs;
	my $distrs = $Analysis::all_distrs;

	# TODO: use selected distrs?
	my @a_distrs = grep {$distrs->{$_}{ARCH} eq $architecture} keys %$distrs;

	my %distr_status = ();
	$distr_status{$_} = 1 for @a_distrs;

	foreach my $Did ( @a_distrs ) {
		foreach my $soname ( keys %$libs ) {
			my $lib = $libs->{$soname};
			if ( !$lib->{DISTRS} || !$lib->{DISTRS}{$Did} ) { # lib is missed in distr
				if ( !$lib->{INTS_USED} || !$lib->{APPROVED} ) {
					if ( $distr_status{$Did} == 1 ) {
						$distr_status{$Did} = 2; # Warning status
					}
				} else {
					$distr_status{$Did} = 0; # Unsupported distr
					last;
				}
			}
		}
	}
	
	my $unsupp_distrs = scalar grep {$_ == 0} values %distr_status;
	my $warn_distrs = scalar grep {$_ == 2} values %distr_status;
	my $good_distrs = scalar grep {$_ == 1} values %distr_status;
	
	my $total_distrs = scalar @a_distrs;
	
	my $lsb_status = "IN";
	my $lsb_status_ver = undef;
	my $unused_nonlsb_libs = 0;

	my $unused_libs = 0;

	foreach my $soname ( keys %$libs ) {
		my $lib = $libs->{$soname};

		# Check libs without interfaces used
		if ( !$lib->{INTS_USED} ) {
			$zero_int_libs++;
		}

		# Check LSB
		my ($status, $status_ver) = lib_lsb_status( $lib, $selected_lsb_ver );
		if ( $status ne "IN" && !$lib->{INTS_USED} ) { # Bad unused lib
			$unused_nonlsb_libs++;
			next;
		}
		if ( $status eq "NOT" ) {
			if ( $status_ver ) {
				if ( $lsb_status eq "FUTURE" ) {
					$lsb_status_ver = $status_ver if $status_ver > $lsb_status_ver;
				}
				elsif ( $lsb_status ne "NOT" ) {
					$lsb_status = "FUTURE";
					$lsb_status_ver = $status_ver;
				}
			} else {
				$lsb_status = "NOT";
				$lsb_status_ver = undef;
			}
		}
		elsif ( $status eq "CANDIDATE" ) {
			if ( $lsb_status eq "CANDIDATE" ) {
				$lsb_status_ver = $status_ver if $status_ver > $lsb_status_ver;
			}
			elsif ( $lsb_status ne "NOT" && $lsb_status ne "FUTURE" ) {
				$lsb_status = "CANDIDATE";
				$lsb_status_ver = $status_ver;
			}
		}
		elsif ( $status eq "OPTIONAL" || $status eq "DEPRECATED" ) {
			if ( $lsb_status eq "IN" ) {
				$lsb_status = "WARN"
			}
			if ( $lsb_status eq "WARN" && $status eq "DEPRECATED" ) {
				if ( !defined $lsb_status_ver || $status_ver < $lsb_status_ver ) {
					$lsb_status_ver = $status_ver;
				}
			}
		}
		elsif ( $status eq "IN" ) {
			if ( $lib->{WITHDRAWNIN} ) { # Withdrawn in further versions
			# TODO: ??? Should warn ???
				if ( $lsb_status eq "IN" ) {
					$lsb_status = "WARN"
				}
				if ( $lsb_status eq "WARN" ) {
					if ( !defined $lsb_status_ver || $status_ver < $lsb_status_ver ) {
						$lsb_status_ver = $status_ver;
					}
				}
			}
		}
	}

	## Let's print the summary

	my $nl = "\n";
	my $res = '';

	my $green_image  = '<img src="images/dot_green.png" alt="green" /> ';
	my $yellow_image = '<img src="images/dot_yellow.png" alt="yellow" /> ';
	my $red_image    = '<img src="images/dot_red.png" alt="red" /> ';

	$res .= '<div id="summary">'.$nl;

	# Summary on compatibility with various distributions
	if ( $warn_distrs == 0 && $unsupp_distrs == 0 ) {
		$res .= '<p>';
		$res .= $green_image; # Green image
		$res .= 'All libraries used by the application are present in all the analyzed distributions.';
		$res .= '</p>';
	}
	elsif ( $warn_distrs > 0 && $unsupp_distrs == 0 ) {
		$res .= '<p>';
		$res .= $green_image; # Yellow image
		$res .= 'All libraries required by the application except some unused or unknown are present in all the analyzed distributions.';
		if ( $_GET{"tab"} ne 'distrs' ) {
			my $link = '?details='.$_GET{'details'}.'&tab=distrs';
			$res .= ' (<a href="'.$link.'">Details&hellip;</a>)';
		}
		$res .= '</p>';
	}
	elsif ( $good_distrs > 0 && $warn_distrs == 0 && $unsupp_distrs > 0 ) {
		$res .= '<p>';
		$res .= $yellow_image; # Yellow image
		$res .= 'There '.($good_distrs > 1 ? 'are' : 'is' ).' '.$good_distrs
				.' of '.$total_distrs.' distributions where all the libraries'
				.' required by the application are present.';
		if ( $_GET{"tab"} ne 'distrs' ) {
			my $link = '?details='.$_GET{'details'}.'&tab=distrs';
			$res .= ' (<a href="'.$link.'">Details&hellip;</a>)';
		}
		$res .= '</p>';
	}
	elsif ( $warn_distrs > 0 && $unsupp_distrs > 0 ) {
		$res .= '<p>';
		$res .= $yellow_image; # Yellow image
		my $n = $good_distrs + $warn_distrs;
		$res .= 'There '.($n > 1 ? 'are' : 'is' ).' '.$n
				.' of '.$total_distrs.' distributions where all the required libraries'
				.' except unused and unknown are present.';
		if ( $_GET{"tab"} ne 'distrs' ) {
			my $link = '?details='.$_GET{'details'}.'&tab=distrs';
			$res .= ' (<a href="'.$link.'">Details&hellip;</a>)';
		}
		$res .= '</p>';
	}
	elsif ( $good_distrs == 0 && $warn_distrs == 0 && $unsupp_distrs > 0 ) {
		$res .= '<p>';
		$res .= $red_image; # Red image
		$res .= 'The application is unlikely to run on any of the analyzed distributions.';
		$res .= '</p>';
	}
	$res .= $nl;

	# Libraries without interfaces used
	if ( $zero_int_libs ) {
		$res .= '<p>';
		$res .= $yellow_image; # Yellow image
		$res .= 'There are <b>'.$zero_int_libs.'</b> '.($zero_int_libs > 1 ? 'libraries' : 'library')
			.' with <b>no interfaces</b> actually used.';
		$res .= ' You\'d better compile your application without those libraries to avoid possible portability problems.';
		$res .= '</p>';
	}
	
	if ( $warn_distrs ) {
		$res .= '<p>';
		$res .= $yellow_image; # Yellow image
		$res .= 'There are some libraries unknown for our analysts. Most likely such libraries are just application specific pieces that should be included as a part of the application.';
		$res .= '</p>';
	}

	# Summary on LSB compatibility
	if ( $lsb_status eq "IN" || ($lsb_status eq "WARN" && !$lsb_status_ver) ) { # IN or OPTIONAL
		$res .= '<p>';
		$res .= $green_image; # Green image
		$res .= 'All libraries used by the application are included in <b>LSB '.$selected_lsb_ver.'</b>.';
		$res .= 'You have good chances to achieve LSB compatibility.';
		$res .= '</p>';
	}
	elsif ( $lsb_status eq "WARN" && $lsb_status_ver ) { # DEPRECATED
		$res .= '<p>';
		$res .= $yellow_image; # Yellow image
		$res .= 'All libraries used by the application are included in <b>LSB '.$selected_lsb_ver.'</b>';
		$res .= ', but some of them are <b>deprecated</b>.';
		$res .= ' You may have compatibility issues with new distributions in future.';
		$res .= '</p>';
	}
	elsif ( $lsb_status eq "FUTURE" ) {
		$res .= '<p>';
		$res .= $yellow_image; # Yellow image
		$res .= 'The Application uses libraries appeared in later version of LSB.';
		$res .= ' Try to check it for compatibility with <b>LSB '.$lsb_status_ver.'</b>';
		$res .= '.</p>';
	}
	elsif ( $lsb_status eq "CANDIDATE" ) {
		$res .= '<p>';
		$res .= $yellow_image; # Yellow image
		$res .= 'All libraries used by the application are either included in LSB '.$selected_lsb_ver.' version or are candidates for LSB '.$lsb_status_ver;
		$res .= '.</p>';
	}
	elsif ( $lsb_status eq "NOT" ) {
		$res .= '<p>';
		$res .= $red_image; # Red image
		$res .= 'The Application uses some libraries <b>incompatible with LSB '.$selected_lsb_ver.'</b>';
		$res .= '.</p>';
	}

	$res .= "$nl</div>$nl";
	$res .= <<DATA;
<br />
DATA

	return $res;
}
